viper

package module
v1.21.0-ak-6 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Jul 23, 2026 License: MIT Imports: 30 Imported by: 0

README

Viper v2 Feedback

Viper is heading towards v2 and we would love to hear what you would like to see in it. Share your thoughts here: https://forms.gle/R6faU74qPRPAzchZ9

Thank you!

viper logo

Mentioned in Awesome Go run on repl.it

GitHub Workflow Status Go Report Card Go Version PkgGoDev

Go configuration with fangs!

Many Go projects are built using Viper including:

Install

go get github.com/alkaid/viper

NOTE Viper uses Go Modules to manage dependencies.

Why use Viper?

Viper is a complete configuration solution for Go applications including 12-Factor apps. It is designed to work within any application, and can handle all types of configuration needs and formats. It supports:

  • setting defaults
  • setting explicit values
  • reading config files
  • dynamic discovery of config files across multiple locations
  • reading from environment variables
  • reading from remote systems (e.g. Etcd or Consul)
  • reading from command line flags
  • reading from buffers
  • live watching and updating configuration
  • aliasing configuration keys for easy refactoring

Viper can be thought of as a registry for all of your applications' configuration needs.

Putting Values in Viper

Viper can read from multiple configuration sources and merges them together into one set of configuration keys and values.

Viper uses the following precedence for merging:

  • explicit call to Set
  • flags
  • environment variables
  • config files
  • external key/value stores
  • defaults

NOTE Viper configuration keys are case insensitive.

Reading Config Files

Viper requires minimal configuration to load config files. Viper currently supports:

  • JSON
  • TOML
  • YAML
  • INI
  • envfile
  • Java Propeties

A single Viper instance only supports a single configuration file, but multiple paths may be searched for one.

Here is an example of how to use Viper to search for and read a configuration file. At least one path should be provided where a configuration file is expected.

// Name of the config file without an extension (Viper will intuit the type
// from an extension on the actual file)
viper.SetConfigName("config")

// Add search paths to find the file
viper.AddConfigPath("/etc/appname/")
viper.AddConfigPath("$HOME/.appname")
viper.AddConfigPath(".")

// Find and read the config file
err := viper.ReadInConfig()

// Handle errors
if err != nil {
	panic(fmt.Errorf("fatal error config file: %w", err))
}

You can handle the specific case where no config file is found.

var fileLookupError viper.FileLookupError
if err := viper.ReadInConfig(); err != nil {
    if errors.As(err, &fileLookupError) {
        // Indicates an explicitly set config file is not found (such as with
        // using `viper.SetConfigFile`) or that no config file was found in
        // any search path (such as when using `viper.AddConfigPath`)
    } else {
        // Config file was found but another error was produced
    }
}

// Config file found and successfully parsed

NOTE (since 1.6) You can also have a file without an extension and specify the format programmatically, which is useful for files that naturally have no extension (e.g., .bashrc).

Writing Config Files

At times you may want to store all configuration modifications made during run time.

// Writes current config to the path set by `AddConfigPath` and `SetConfigName`
viper.WriteConfig()
viper.SafeWriteConfig() // Like the above, but will error if the config file exists

// Writes current config to a specific place
viper.WriteConfigAs("/path/to/my/.config")

// Will error since it has already been written
viper.SafeWriteConfigAs("/path/to/my/.config")

viper.SafeWriteConfigAs("/path/to/my/.other_config")

As a rule of the thumb, methods prefixed with Safe won't overwrite any existing file, while other methods will.

Watching and Re-reading Config Files

Gone are the days of needing to restart a server to have a config take effect--Viper powered applications can read an update to a config file while running and not miss a beat.

It's also possible to provide a function for Viper to run each time a change occurs.

// All config paths must be defined prior to calling `WatchConfig()`
viper.AddConfigPath("$HOME/.appname")

viper.OnConfigChange(func(e fsnotify.Event) {
	fmt.Println("Config file changed:", e.Name)
})

viper.WatchConfig()

Reading Config from io.Reader

Viper predefines many configuration sources but you can also implement your own required configuration source.

viper.SetConfigType("yaml")

var yamlExample = []byte(`
hacker: true
hobbies:
- skateboarding
- snowboarding
- go
name: steve
`)

viper.ReadConfig(bytes.NewBuffer(yamlExample))

viper.Get("name") // "steve"

Setting Defaults

A good configuration system will support default values, which are used if a key hasn't been set in some other way.

Examples:

viper.SetDefault("ContentDir", "content")
viper.SetDefault("LayoutDir", "layouts")
viper.SetDefault("Taxonomies", map[string]string{"tag": "tags", "category": "categories"})

Setting Overrides

Viper allows explict setting of configuration, such as from your own application logic.

viper.Set("verbose", true)
viper.Set("host.port", 5899) // Set an embedded key

Registering and Using Aliases

Aliases permit a single value to be referenced by multiple keys

viper.RegisterAlias("loud", "Verbose")

viper.Set("verbose", true) // Same result as next line
viper.Set("loud", true)    // Same result as prior line

viper.GetBool("loud")    // true
viper.GetBool("verbose") // true

Working with Environment Variables

Viper has full support for environment variables.

NOTE Unlike other configuration sources, environment variables are case sensitive.

// Tells Viper to use this prefix when reading environment variables
viper.SetEnvPrefix("spf")

// Viper will look for "SPF_ID", automatically uppercasing the prefix and key
viper.BindEnv("id")

// Alternatively, we can search for any environment variable prefixed and load
// them in
viper.AutomaticEnv()

os.Setenv("SPF_ID", "13")

id := viper.Get("id") // 13
  • By default, empty environment variables are considered unset and will fall back to the next configuration source, unless AllowEmptyEnv is used.
  • Viper does not "cache" environment variables--the value will be read each time it is accessed.
  • SetEnvKeyReplacer and EnvKeyReplacer allow you to rewrite environment variable keys, which is useful to merge SCREAMING_SNAKE_CASE environment variables with kebab-cased configuration values from other sources.

Working with Flags

Viper has the ability to bind to flags. Specifically, Viper supports pflag as used in the Cobra library.

Like environment variables, the value is not set when the binding method is called, but when it is accessed.

For individual flags, the BindPFlag method provides this functionality.

serverCmd.Flags().Int("port", 1138, "Port to run Application server on")

viper.BindPFlag("port", serverCmd.Flags().Lookup("port"))

You can also bind an existing set of pflags.

pflag.Int("flagname", 1234, "help message for flagname")
pflag.Parse()

viper.BindPFlags(pflag.CommandLine)

i := viper.GetInt("flagname") // Retrieve values from viper instead of pflag

The standard library flag package is not directly supported, but may be parsed through pflag.

package main

import (
	"flag"

	"github.com/spf13/pflag"
)

func main() {
	// Using standard library "flag" package
	flag.Int("flagname", 1234, "help message for flagname")

    // Pass standard library flags to pflag
	pflag.CommandLine.AddGoFlagSet(flag.CommandLine)
	pflag.Parse()

    // Viper takes over
	viper.BindPFlags(pflag.CommandLine)
}

Use of pflag may be avoided entirely by implementing the FlagValue and FlagValueSet interfaces.

// Implementing FlagValue

type myFlag struct {}
func (f myFlag) HasChanged() bool { return false }
func (f myFlag) Name() string { return "my-flag-name" }
func (f myFlag) ValueString() string { return "my-flag-value" }
func (f myFlag) ValueType() string { return "string" }

viper.BindFlagValue("my-flag-name", myFlag{})

// Implementing FlagValueSet

type myFlagSet struct {
	flags []myFlag
}
func (f myFlagSet) VisitAll(fn func(FlagValue)) {
	for _, flag := range flags {
		fn(flag)
	}
}

fSet := myFlagSet{
	flags: []myFlag{myFlag{}, myFlag{}},
}
viper.BindFlagValues("my-flags", fSet)

Remote Key/Value Store Support

To enable remote support in Viper, do a blank import of the viper/remote package.

import _ "github.com/alkaid/viper/remote"

Viper supports the following remote key/value stores. Examples for each are provided below.

  • Etcd and Etcd3
  • Consul
  • Firestore
  • NATS

Viper will read a config string retrieved from a path in a key/value store.

Viper supports multiple hosts separated by ;. For example: http://127.0.0.1:4001;http://127.0.0.1:4002.

Encryption

Viper uses crypt to retrieve configuration from the key/value store, which means that you can store your configuration values encrypted and have them automatically decrypted if you have the correct GPG keyring. Encryption is optional.

Crypt has a command-line helper that you can use to put configurations in your key/value store.

$ go get github.com/alkaid/crypt/bin/crypt
$ crypt set -plaintext /config/hugo.json /Users/hugo/settings/config.json
$ crypt get -plaintext /config/hugo.json

See the Crypt documentation for examples of how to set encrypted values, or how to use Consul.

Remote Key/Value Store Examples (Unencrypted)

etcd
viper.AddRemoteProvider("etcd", "http://127.0.0.1:4001","/config/hugo.json")
viper.SetConfigType("json") // because there is no file extension in a stream of bytes, supported extensions are "json", "toml", "yaml", "yml", "properties", "props", "prop", "env", "dotenv"
err := viper.ReadRemoteConfig()
etcd3
viper.AddRemoteProvider("etcd3", "http://127.0.0.1:4001","/config/hugo.json")
viper.SetConfigType("json") // because there is no file extension in a stream of bytes, supported extensions are "json", "toml", "yaml", "yml", "properties", "props", "prop", "env", "dotenv"
err := viper.ReadRemoteConfig()
Consul

Given a Consul key MY_CONSUL_KEY with the value:

{
    "port": 8080,
    "hostname": "myhostname.com"
}
viper.AddRemoteProvider("consul", "localhost:8500", "MY_CONSUL_KEY")
viper.SetConfigType("json") // Need to explicitly set this to json
err := viper.ReadRemoteConfig()

fmt.Println(viper.Get("port")) // 8080
Firestore
viper.AddRemoteProvider("firestore", "google-cloud-project-id", "collection/document")
viper.SetConfigType("json") // Config's format: "json", "toml", "yaml", "yml"
err := viper.ReadRemoteConfig()

Of course, you're allowed to use SecureRemoteProvider also.

NATS
viper.AddRemoteProvider("nats", "nats://127.0.0.1:4222", "myapp.config")
viper.SetConfigType("json")
err := viper.ReadRemoteConfig()

Remote Key/Value Store Examples (Encrypted)

viper.AddSecureRemoteProvider("etcd","http://127.0.0.1:4001","/config/hugo.json","/etc/secrets/mykeyring.gpg")
viper.SetConfigType("json") // because there is no file extension in a stream of bytes,  supported extensions are "json", "toml", "yaml", "yml", "properties", "props", "prop", "env", "dotenv"
err := viper.ReadRemoteConfig()

Watching Key/Value Store Changes

// Alternatively, you can create a new viper instance
var runtime_viper = viper.New()

runtime_viper.AddRemoteProvider("etcd", "http://127.0.0.1:4001", "/config/hugo.yml")
runtime_viper.SetConfigType("yaml") // because there is no file extension in a stream of bytes, supported extensions are "json", "toml", "yaml", "yml", "properties", "props", "prop", "env", "dotenv"

// Read from remote config the first time
err := runtime_viper.ReadRemoteConfig()

// Unmarshal config
runtime_viper.Unmarshal(&runtime_conf)

// Open a goroutine to watch remote changes forever
go func(){
	for {
		time.Sleep(time.Second * 5) // delay after each request

		// Currently, only tested with Etcd support
		err := runtime_viper.WatchRemoteConfig()
		if err != nil {
			log.Errorf("unable to read remote config: %v", err)
			continue
		}

		// Unmarshal new config into our runtime config struct
		runtime_viper.Unmarshal(&runtime_conf)
	}
}()

Getting Values From Viper

The simplest way to retrieve configuration values from Viper is to use Get* functions. Get will return an any type, but specific types may be retrieved with Get<Type> functions.

Note that each Get* function will return a zero value if it’s key is not found. To check if a key exists, use the IsSet method.

Nested keys use . as a delimiter and numbers for array indexes. Given the following configuration:

{
    "datastore": {
        "metric": {
            "host": "127.0.0.1",
            "ports": [
                5799,
                6029
            ]
        }
    }
}

GetString("datastore.metric.host") // "127.0.0.1"
GetInt("host.ports.1") // 6029

NOTE Viper does not deep merge configuration values. Complex values that are overridden will be entirely replaced.

If there exists a key that matches the delimited key path, its value will be returned instead.

{
    "datastore.metric.host": "0.0.0.0",
    "datastore": {
        "metric": {
            "host": "127.0.0.1"
        }
    }
}
GetString("datastore.metric.host") // "0.0.0.0"

Configuration Subsets

It's often useful to extract a subset of configuration (e.g., when developing a reusable module which should accept specific sections of configuration).

cache:
  cache1:
    item-size: 64
    max-items: 100
  cache2:
    item-size: 80
    max-items: 200
func NewCache(v *Viper) *Cache {
	return &Cache{
		ItemSize: v.GetInt("item-size"),
		MaxItems: v.GetInt("max-items"),
	}
}

cache1Config := viper.Sub("cache.cache1")

if cache1Config == nil {
    // Sub returns nil if the key cannot be found
	panic("cache configuration not found")
}

cache1 := NewCache(cache1Config)

Unmarshaling

You also have the option of unmarshaling configuration to a struct, map, etc., using Unmarshal* methods.

type config struct {
	Port int
	Name string
	PathMap string `mapstructure:"path_map"`
}

var C config

err := viper.Unmarshal(&C)
if err != nil {
	t.Fatalf("unable to decode into struct, %v", err)
}

If you want to unmarshal configuration where the keys themselves contain . (the default key delimiter), you can change the delimiter.

v := viper.NewWithOptions(viper.KeyDelimiter("::"))

v.SetDefault("chart::values", map[string]any{
	"ingress": map[string]any{
		"annotations": map[string]any{
			"traefik.frontend.rule.type":                 "PathPrefix",
			"traefik.ingress.kubernetes.io/ssl-redirect": "true",
		},
	},
})

type config struct {
	Chart struct{
		Values map[string]any
	}
}

var C config

v.Unmarshal(&C)

Viper also supports unmarshaling into embedded structs.

/*
Example config:

module:
    enabled: true
    token: 89h3f98hbwf987h3f98wenf89ehf
*/
type config struct {
	Module struct {
		Enabled bool

		moduleConfig `mapstructure:",squash"`
	}
}

type moduleConfig struct {
	Token string
}

var C config

err := viper.Unmarshal(&C)
if err != nil {
	t.Fatalf("unable to decode into struct, %v", err)
}

Viper uses github.com/go-viper/mapstructure under the hood for unmarshaling values which uses mapstructure tags, by default.

Marshalling to String

You may need to marshal all the settings held in Viper into a string. You can use your favorite format's marshaller with the config returned by AllSettings.

import (
	yaml "go.yaml.in/yaml/v3"
)

func yamlStringSettings() string {
	c := viper.AllSettings()
	bs, err := yaml.Marshal(c)
	if err != nil {
		log.Fatalf("unable to marshal config to YAML: %v", err)
	}
	return string(bs)
}

Decoding Custom Formats

A frequently requested feature is adding more value formats and decoders (for example; parsing character delimited strings into slices. This is already available in Viper using mapstructure decode hooks.

Read more in this blog post.

FAQ

Why is it called “Viper”?

Viper is designed to be a companion to Cobra. While both can operate completely independently, together they make a powerful pair to handle much of your application foundation needs.

I found a bug or want a feature, should I file an issue or a PR?

Yes, but there are two things to be aware of.

  1. The Viper project is currently prioritizing backwards compatibility and stability over features.
  2. Features may be deferred until Viper 2 forms.

Can multiple Viper instances be used?

tl;dr: Yes.

Each will have its own unique configuration and can read from a different configuration source. All of the functions that the Viper package supports are mirrored as methods on a Viper instance.

x := viper.New()
y := viper.New()

x.SetDefault("ContentDir", "content")
y.SetDefault("ContentDir", "foobar")

Should Viper be a global singleton or passed around?

The best practice is to initialize a Viper instance and pass that around when necessary.

Viper comes with a global instance (singleton) out of the box. Although it makes setting up configuration easy, using it is generally discouraged as it makes testing harder and can lead to unexpected behavior.

The global instance may be deprecated in the future. See #1855 for more details.

Does Viper support case sensitive keys?

tl;dr: No.

Viper merges configuration from various sources, many of which are either case insensitive or use different casing than other sources (e.g., env vars). In order to provide the best experience when using multiple sources, all keys are made case insensitive.

There has been several attempts to implement case sensitivity, but unfortunately it's not trivial. We might take a stab at implementing it in Viper v2, but despite the initial noise, it does not seem to be requested that much.

You can vote for case sensitivity by filling out this feedback form: https://forms.gle/R6faU74qPRPAzchZ9.

Is it safe to concurrently read and write to a Viper instance?

No, you will need to synchronize access to Viper yourself (for example by using the sync package). Concurrent reads and writes can cause a panic.

Troubleshooting

See TROUBLESHOOTING.md.

Development

For an optimal developer experience, it is recommended to install Nix and direnv.

Alternatively, install Go on your computer then run make deps to install the rest of the dependencies.

Run the test suite:

make test

Run linters:

make lint # pass -j option to run them in parallel

Some linter violations can automatically be fixed:

make fmt

License

The project is licensed under the MIT License.

Documentation

Index

Examples

Constants

This section is empty.

Variables

var RemoteConfig remoteConfigFactory

RemoteConfig is optional, see the remote package.

var SupportedExts = []string{"json", "toml", "yaml", "yml", "properties", "props", "prop", "hcl", "tfvars", "dotenv", "env", "ini"}

SupportedExts are universally supported extensions.

var SupportedRemoteProviders = []string{"etcd", "etcd3", "consul", "firestore", "nats"}

SupportedRemoteProviders are universally supported remote providers.

Functions

func AddConfigPath

func AddConfigPath(in string)

AddConfigPath adds a path for Viper to search for the config file in. Can be called multiple times to define multiple search paths.

func AddRemoteProvider

func AddRemoteProvider(provider, endpoint, path string) error

AddRemoteProvider adds a remote configuration source. Remote providers are merged in the order they are added. Providers added later have higher priority. provider is a string value: "etcd", "etcd3", "consul", "firestore" or "nats" are currently supported. endpoint is the url. etcd requires http://ip:port, consul requires ip:port, nats requires nats://ip:port path is the path in the k/v store to retrieve configuration To retrieve a config file called myapp.json from /configs/myapp.json you should set path to /configs and set config name (SetConfigName()) to "myapp".

func AddSecureRemoteProvider

func AddSecureRemoteProvider(provider, endpoint, path, secretkeyring string) error

AddSecureRemoteProvider adds a remote configuration source. Secure remote providers are merged in the order they are added. Providers added later have higher priority. provider is a string value: "etcd", "etcd3", "consul", "firestore" or "nats" are currently supported. endpoint is the url. etcd requires http://ip:port consul requires ip:port secretkeyring is the filepath to your openpgp secret keyring. e.g. /etc/secrets/myring.gpg path is the path in the k/v store to retrieve configuration To retrieve a config file called myapp.json from /configs/myapp.json you should set path to /configs and set config name (SetConfigName()) to "myapp". Secure Remote Providers are implemented with github.com/alkaid/crypt.

func AllKeys

func AllKeys() []string

AllKeys returns all keys holding a value, regardless of where they are set. Nested keys are returned with a v.keyDelim separator.

func AllSettings

func AllSettings() map[string]any

AllSettings merges all settings and returns them as a map[string]any.

func AllowEmptyEnv

func AllowEmptyEnv(allowEmptyEnv bool)

AllowEmptyEnv tells Viper to consider set, but empty environment variables as valid values instead of falling back. For backward compatibility reasons this is false by default.

func AutomaticEnv

func AutomaticEnv()

AutomaticEnv makes Viper check if environment variables match any of the existing keys (config, default or flags). If matching env vars are found, they are loaded into Viper.

func BindEnv

func BindEnv(input ...string) error

BindEnv binds a Viper key to a ENV variable. ENV variables are case sensitive. If only a key is provided, it will use the env key matching the key, uppercased. If more arguments are provided, they will represent the env variable names that should bind to this key and will be taken in the specified order. EnvPrefix will be used when set when env name is not provided.

func BindFlagValue

func BindFlagValue(key string, flag FlagValue) error

BindFlagValue binds a specific key to a FlagValue.

func BindFlagValues

func BindFlagValues(flags FlagValueSet) error

BindFlagValues binds a full FlagValue set to the configuration, using each flag's long name as the config key.

func BindPFlag

func BindPFlag(key string, flag *pflag.Flag) error

BindPFlag binds a specific key to a pflag (as used by cobra). Example (where serverCmd is a Cobra instance):

serverCmd.Flags().Int("port", 1138, "Port to run Application server on")
Viper.BindPFlag("port", serverCmd.Flags().Lookup("port"))

func BindPFlags

func BindPFlags(flags *pflag.FlagSet) error

BindPFlags binds a full flag set to the configuration, using each flag's long name as the config key.

func ConfigFileUsed

func ConfigFileUsed() string

ConfigFileUsed returns the file used to populate the config registry.

func Debug

func Debug()

Debug prints all configuration registries for debugging purposes.

func DebugTo

func DebugTo(w io.Writer)

DebugTo writes all configuration registries to the given io.Writer.

func Get

func Get(key string) any

Get can retrieve any value given the key to use. Get is case-insensitive for a key. Get has the behavior of returning the value associated with the first place from where it is set. Viper will check in the following order: override, flag, env, config file, key/value store, default

Get returns an interface. For a specific value use one of the Get____ methods.

func GetBool

func GetBool(key string) bool

GetBool returns the value associated with the key as a boolean.

func GetDuration

func GetDuration(key string) time.Duration

GetDuration returns the value associated with the key as a duration.

func GetEnvPrefix

func GetEnvPrefix() string

GetEnvPrefix returns the environment variable prefix.

func GetFloat64

func GetFloat64(key string) float64

GetFloat64 returns the value associated with the key as a float64.

func GetInt

func GetInt(key string) int

GetInt returns the value associated with the key as an integer.

func GetInt32 added in v1.1.0

func GetInt32(key string) int32

GetInt32 returns the value associated with the key as an integer.

func GetInt64

func GetInt64(key string) int64

GetInt64 returns the value associated with the key as an integer.

func GetIntSlice

func GetIntSlice(key string) []int

GetIntSlice returns the value associated with the key as a slice of int values.

func GetSizeInBytes

func GetSizeInBytes(key string) uint

GetSizeInBytes returns the size of the value associated with the given key in bytes.

func GetString

func GetString(key string) string

GetString returns the value associated with the key as a string.

func GetStringMap

func GetStringMap(key string) map[string]any

GetStringMap returns the value associated with the key as a map of interfaces.

func GetStringMapString

func GetStringMapString(key string) map[string]string

GetStringMapString returns the value associated with the key as a map of strings.

func GetStringMapStringSlice

func GetStringMapStringSlice(key string) map[string][]string

GetStringMapStringSlice returns the value associated with the key as a map to a slice of strings.

func GetStringSlice

func GetStringSlice(key string) []string

GetStringSlice returns the value associated with the key as a slice of strings.

func GetTime

func GetTime(key string) time.Time

GetTime returns the value associated with the key as time.

func GetUint

func GetUint(key string) uint

GetUint returns the value associated with the key as an unsigned integer.

func GetUint8

func GetUint8(key string) uint8

GetUint8 returns the value associated with the key as an unsigned integer.

func GetUint16

func GetUint16(key string) uint16

GetUint16 returns the value associated with the key as an unsigned integer.

func GetUint32

func GetUint32(key string) uint32

GetUint32 returns the value associated with the key as an unsigned integer.

func GetUint64

func GetUint64(key string) uint64

GetUint64 returns the value associated with the key as an unsigned integer.

func InConfig

func InConfig(key string) bool

InConfig checks to see if the given key (or an alias) is in the config file.

func IsSet

func IsSet(key string) bool

IsSet checks to see if the key has been set in any of the data locations. IsSet is case-insensitive for a key.

func MergeConfig

func MergeConfig(in io.Reader) error

MergeConfig merges a new configuration with an existing config.

func MergeConfigMap

func MergeConfigMap(cfg map[string]any) error

MergeConfigMap merges the configuration from the map given with an existing config. Note that the map given may be modified.

func MergeInConfig

func MergeInConfig() error

MergeInConfig merges a new configuration with an existing config.

func MustBindEnv

func MustBindEnv(input ...string)

MustBindEnv wraps BindEnv in a panic. If there is an error binding an environment variable, MustBindEnv will panic.

func OnConfigChange

func OnConfigChange(run func(in fsnotify.Event))

OnConfigChange sets the event handler that is called when a config file changes.

func ReadConfig

func ReadConfig(in io.Reader) error

ReadConfig will read a configuration file, setting existing keys to nil if the key does not exist in the file.

func ReadInConfig

func ReadInConfig() error

ReadInConfig will discover and load the configuration file from disk and key/value stores, searching in one of the defined paths.

func ReadRemoteConfig

func ReadRemoteConfig() error

ReadRemoteConfig reads and merges all registered remote configuration sources. Providers added later override earlier providers.

func ReadRemoteConfigWithMerged

func ReadRemoteConfigWithMerged(merged bool) error

ReadRemoteConfigWithMerged reads and optionally deep-merges all remote configuration sources.

func RegisterAlias

func RegisterAlias(alias, key string)

RegisterAlias creates an alias that provides another accessor for the same key. This enables one to change a name without breaking the application.

func Reset

func Reset()

Reset is intended for testing, will reset all to default settings. In the public interface for the viper package so applications can use it in their testing as well.

func SafeWriteConfig added in v1.0.1

func SafeWriteConfig() error

SafeWriteConfig writes current configuration to file only if the file does not exist.

func SafeWriteConfigAs added in v1.0.1

func SafeWriteConfigAs(filename string) error

SafeWriteConfigAs writes current configuration to a given filename if it does not exist.

func Set

func Set(key string, value any)

Set sets the value for the key in the override register. Set is case-insensitive for a key. Will be used instead of values obtained via flags, config file, ENV, default, or key/value store.

func SetConfigFile

func SetConfigFile(in string)

SetConfigFile explicitly defines the path, name and extension of the config file. Viper will use this and not check any of the config paths.

func SetConfigName

func SetConfigName(in string)

SetConfigName sets name for the config file. Does not include extension.

func SetConfigPermissions

func SetConfigPermissions(perm os.FileMode)

SetConfigPermissions sets the permissions for the config file.

func SetConfigType

func SetConfigType(in string)

SetConfigType sets the type of the configuration returned by the remote source, e.g. "json".

func SetDefault

func SetDefault(key string, value any)

SetDefault sets the default value for this key. SetDefault is case-insensitive for a key. Default only used when no value is provided by the user via flag, config or ENV.

func SetEnvKeyReplacer

func SetEnvKeyReplacer(r *strings.Replacer)

SetEnvKeyReplacer sets the strings.Replacer on the viper object Useful for mapping an environmental variable to a key that does not match it.

func SetEnvPrefix

func SetEnvPrefix(in string)

SetEnvPrefix defines a prefix that ENVIRONMENT variables will use. E.g. if your prefix is "spf", the env registry will look for env variables that start with "SPF_".

func SetFs

func SetFs(fs afero.Fs)

SetFs sets the filesystem to use to read configuration.

func SetOptions

func SetOptions(opts ...Option)

SetOptions sets the options on the global Viper instance.

Be careful when using this function: subsequent calls may override options you set. It's always better to use a local Viper instance.

func SetTypeByDefaultValue

func SetTypeByDefaultValue(enable bool)

SetTypeByDefaultValue enables or disables the inference of a key value's type when the Get function is used based upon a key's default value as opposed to the value returned based on the normal fetch logic.

For example, if a key has a default value of []string{} and the same key is set via an environment variable to "a b c", a call to the Get function would return a string slice for the key if the key's type is inferred by the default value and the Get function would return:

[]string {"a", "b", "c"}

Otherwise the Get function would return:

"a b c"

func Unmarshal

func Unmarshal(rawVal any, opts ...DecoderConfigOption) error

Unmarshal unmarshals the config into a Struct. Make sure that the tags on the fields of the structure are properly set.

func UnmarshalExact

func UnmarshalExact(rawVal any, opts ...DecoderConfigOption) error

UnmarshalExact unmarshals the config into a Struct, erroring if a field is nonexistent in the destination struct.

func UnmarshalKey

func UnmarshalKey(key string, rawVal any, opts ...DecoderConfigOption) error

UnmarshalKey takes a single key and unmarshals it into a Struct.

func WatchConfig

func WatchConfig()

WatchConfig starts watching a config file for changes.

func WatchRemoteConfig

func WatchRemoteConfig() error

WatchRemoteConfig updates configuration from available remote providers.

func WriteConfig added in v1.0.1

func WriteConfig() error

WriteConfig writes the current configuration to a file.

func WriteConfigAs added in v1.0.1

func WriteConfigAs(filename string) error

WriteConfigAs writes current configuration to a given filename.

func WriteConfigTo

func WriteConfigTo(w io.Writer) error

WriteConfigTo writes current configuration to an io.Writer.

Types

type Codec

type Codec interface {
	Encoder
	Decoder
}

Codec combines Encoder and Decoder interfaces.

type CodecRegistry

type CodecRegistry interface {
	EncoderRegistry
	DecoderRegistry
}

CodecRegistry combines EncoderRegistry and DecoderRegistry interfaces.

type ConfigFileAlreadyExistsError

type ConfigFileAlreadyExistsError string

ConfigFileAlreadyExistsError denotes failure to write new configuration file.

func (ConfigFileAlreadyExistsError) Error

Error returns the formatted error when configuration already exists.

type ConfigFileNotFoundError deprecated

type ConfigFileNotFoundError struct {
	// contains filtered or unexported fields
}

ConfigFileNotFoundError denotes failing to find a configuration file from a search.

Deprecated: This is error wraps FileNotFoundFromSearchError, which should be used instead.

func (ConfigFileNotFoundError) Error

func (e ConfigFileNotFoundError) Error() string

Error returns the formatted error.

func (ConfigFileNotFoundError) Unwrap

func (e ConfigFileNotFoundError) Unwrap() error

Unwraps to FileNotFoundFromSearchError.

type ConfigMarshalError added in v1.0.1

type ConfigMarshalError struct {
	// contains filtered or unexported fields
}

ConfigMarshalError happens when failing to marshal the configuration.

func (ConfigMarshalError) Error added in v1.0.1

func (e ConfigMarshalError) Error() string

Error returns the formatted configuration error.

type ConfigParseError

type ConfigParseError struct {
	// contains filtered or unexported fields
}

ConfigParseError denotes failing to parse configuration file.

func (ConfigParseError) Error

func (pe ConfigParseError) Error() string

Error returns the formatted configuration error.

func (ConfigParseError) Unwrap

func (pe ConfigParseError) Unwrap() error

Unwrap returns the wrapped error.

type Decoder

type Decoder interface {
	Decode(b []byte, v map[string]any) error
}

Decoder decodes the contents of a byte slice into Viper's internal data structures. It's primarily used for decoding contents of a file into a map[string]any.

type DecoderConfigOption added in v1.1.0

type DecoderConfigOption func(*mapstructure.DecoderConfig)

A DecoderConfigOption can be passed to viper.Unmarshal to configure mapstructure.DecoderConfig options.

func DecodeHook added in v1.1.0

DecodeHook returns a DecoderConfigOption which overrides the default DecoderConfig.DecodeHook value, the default is:

 mapstructure.ComposeDecodeHookFunc(
		mapstructure.StringToTimeDurationHookFunc(),
		mapstructure.StringToSliceHookFunc(","),
	)

type DecoderRegistry

type DecoderRegistry interface {
	Decoder(format string) (Decoder, error)
}

DecoderRegistry returns an Decoder for a given format.

Format is case-insensitive.

DecoderRegistry returns an error if no Decoder is registered for the format.

type DefaultCodecRegistry

type DefaultCodecRegistry struct {
	// contains filtered or unexported fields
}

DefaultCodecRegistry is a simple implementation of CodecRegistry that allows registering custom [Codec]s.

func NewCodecRegistry

func NewCodecRegistry() *DefaultCodecRegistry

NewCodecRegistry returns a new CodecRegistry, ready to accept custom [Codec]s.

func (*DefaultCodecRegistry) Decoder

func (r *DefaultCodecRegistry) Decoder(format string) (Decoder, error)

Decoder implements the DecoderRegistry interface.

Format is case-insensitive.

func (*DefaultCodecRegistry) Encoder

func (r *DefaultCodecRegistry) Encoder(format string) (Encoder, error)

Encoder implements the EncoderRegistry interface.

Format is case-insensitive.

func (*DefaultCodecRegistry) RegisterCodec

func (r *DefaultCodecRegistry) RegisterCodec(format string, codec Codec) error

RegisterCodec registers a custom Codec.

Format is case-insensitive.

type Encoder

type Encoder interface {
	Encode(v map[string]any) ([]byte, error)
}

Encoder encodes Viper's internal data structures into a byte representation. It's primarily used for encoding a map[string]any into a file format.

type EncoderRegistry

type EncoderRegistry interface {
	Encoder(format string) (Encoder, error)
}

EncoderRegistry returns an Encoder for a given format.

Format is case-insensitive.

EncoderRegistry returns an error if no Encoder is registered for the format.

type FileLookupError

type FileLookupError interface {
	error
	// contains filtered or unexported methods
}

FileLookupError is returned when Viper cannot resolve a configuration file.

This is meant to be a common interface for all file look-up errors, occurring either because a file does not exist or because it cannot find any file matching finder criteria.

type FileNotFoundError

type FileNotFoundError struct {
	// contains filtered or unexported fields
}

FileNotFoundError denotes failing to find a specific configuration file.

func (FileNotFoundError) Error

func (e FileNotFoundError) Error() string

Error returns the formatted error.

type FileNotFoundFromSearchError

type FileNotFoundFromSearchError struct {
	// contains filtered or unexported fields
}

FileNotFoundFromSearchError denotes failing to find a configuration file from a search. Wraps ConfigFileNotFoundError.

func (FileNotFoundFromSearchError) Error

Error returns the formatted error.

type Finder

type Finder interface {
	Find(fsys afero.Fs) ([]string, error)
}

Finder looks for files and directories in an afero.Fs filesystem.

Example
package main

import (
	"fmt"

	"github.com/sagikazarmark/locafero"
	"github.com/spf13/afero"

	"github.com/alkaid/viper"
)

func main() {
	fs := afero.NewMemMapFs()

	fs.Mkdir("/home/user", 0o777)

	f, _ := fs.Create("/home/user/myapp.yaml")
	f.WriteString("foo: bar")
	f.Close()

	// HCL will have a "lower" priority in the search order
	fs.Create("/home/user/myapp.hcl")

	finder := locafero.Finder{
		Paths: []string{"/home/user"},
		Names: locafero.NameWithExtensions("myapp", viper.SupportedExts...),
		Type:  locafero.FileTypeFile, // This is important!
	}

	v := viper.NewWithOptions(viper.WithFinder(finder))
	v.SetFs(fs)
	v.ReadInConfig()

	fmt.Println(v.GetString("foo"))

}
Output:
bar

func Finders

func Finders(finders ...Finder) Finder

Finders combines multiple finders into one.

Example
package main

import (
	"fmt"

	"github.com/sagikazarmark/locafero"
	"github.com/spf13/afero"

	"github.com/alkaid/viper"
)

func main() {
	fs := afero.NewMemMapFs()

	fs.Mkdir("/home/user", 0o777)
	f, _ := fs.Create("/home/user/myapp.yaml")
	f.WriteString("foo: bar")
	f.Close()

	fs.Mkdir("/etc/myapp", 0o777)
	fs.Create("/etc/myapp/config.yaml")

	// Combine multiple finders to search for files in multiple locations with different criteria
	finder := viper.Finders(
		locafero.Finder{
			Paths: []string{"/home/user"},
			Names: locafero.NameWithExtensions("myapp", viper.SupportedExts...),
			Type:  locafero.FileTypeFile, // This is important!
		},
		locafero.Finder{
			Paths: []string{"/etc/myapp"},
			Names: []string{"config.yaml"}, // Only accept YAML files in the system config directory
			Type:  locafero.FileTypeFile,   // This is important!
		},
	)

	v := viper.NewWithOptions(viper.WithFinder(finder))
	v.SetFs(fs)
	v.ReadInConfig()

	fmt.Println(v.GetString("foo"))

}
Output:
bar

type FlagValue

type FlagValue interface {
	HasChanged() bool
	Name() string
	ValueString() string
	ValueType() string
}

FlagValue is an interface that users can implement to bind different flags to viper.

type FlagValueSet

type FlagValueSet interface {
	VisitAll(fn func(FlagValue))
}

FlagValueSet is an interface that users can implement to bind a set of flags to viper.

type Option

type Option interface {
	// contains filtered or unexported methods
}

Option configures Viper using the functional options paradigm popularized by Rob Pike and Dave Cheney. If you're unfamiliar with this style, see https://commandcenter.blogspot.com/2014/01/self-referential-functions-and-design.html and https://dave.cheney.net/2014/10/17/functional-options-for-friendly-apis.

func EnvKeyReplacer

func EnvKeyReplacer(r StringReplacer) Option

EnvKeyReplacer sets a replacer used for mapping environment variables to internal keys.

func ExperimentalBindStruct

func ExperimentalBindStruct() Option

ExperimentalBindStruct tells Viper to use the new bind struct feature.

func ExperimentalFinder

func ExperimentalFinder() Option

ExperimentalFinder tells Viper to use the new Finder interface for finding configuration files.

func KeyDelimiter

func KeyDelimiter(d string) Option

KeyDelimiter sets the delimiter used for determining key parts. By default it's value is ".".

func WithCodecRegistry

func WithCodecRegistry(r CodecRegistry) Option

WithCodecRegistry sets a custom EncoderRegistry and DecoderRegistry.

func WithDecodeHook

func WithDecodeHook(h mapstructure.DecodeHookFunc) Option

WithDecodeHook sets a default decode hook for mapstructure.

func WithDecoderRegistry

func WithDecoderRegistry(r DecoderRegistry) Option

WithDecoderRegistry sets a custom DecoderRegistry.

func WithEncoderRegistry

func WithEncoderRegistry(r EncoderRegistry) Option

WithEncoderRegistry sets a custom EncoderRegistry.

func WithFinder

func WithFinder(f Finder) Option

WithFinder sets a custom Finder.

func WithLogger

func WithLogger(l *slog.Logger) Option

WithLogger sets a custom logger.

type RemoteConfigError

type RemoteConfigError string

RemoteConfigError denotes encountering an error while trying to pull the configuration from the remote provider.

func (RemoteConfigError) Error

func (rce RemoteConfigError) Error() string

Error returns the formatted remote provider error.

type RemoteProvider

type RemoteProvider interface {
	Provider() string
	Endpoint() string
	Endpoints() []string
	Path() string
	SecretKeyring() string
}

RemoteProvider stores the configuration necessary to connect to a remote key/value store. Optional secretKeyring to unencrypt encrypted values can be provided.

type RemoteResponse

type RemoteResponse struct {
	Value []byte
	Error error
}

RemoteResponse represents a response from a remote configuration provider.

type StringReplacer

type StringReplacer interface {
	// Replace returns a copy of s with all replacements performed.
	Replace(s string) string
}

StringReplacer applies a set of replacements to a string.

type UnsupportedConfigError

type UnsupportedConfigError string

UnsupportedConfigError denotes encountering an unsupported configuration filetype.

func (UnsupportedConfigError) Error

func (str UnsupportedConfigError) Error() string

Error returns the formatted configuration error.

type UnsupportedRemoteProviderError

type UnsupportedRemoteProviderError string

UnsupportedRemoteProviderError denotes encountering an unsupported remote provider. Currently only etcd and Consul are supported.

func (UnsupportedRemoteProviderError) Error

Error returns the formatted remote provider error.

type Viper

type Viper struct {
	// contains filtered or unexported fields
}

Viper is a prioritized configuration registry. It maintains a set of configuration sources, fetches values to populate those, and provides them according to the source's priority. The priority of the sources is the following: 1. overrides 2. flags 3. env. variables 4. config file 5. key/value store 6. defaults

For example, if values from the following sources were loaded:

Defaults : {
	"secret": "",
	"user": "default",
	"endpoint": "https://localhost"
}
Config : {
	"user": "root"
	"secret": "defaultsecret"
}
Env : {
	"secret": "somesecretkey"
}

The resulting config will have the following values:

{
	"secret": "somesecretkey",
	"user": "root",
	"endpoint": "https://localhost"
}

Note: Vipers are not safe for concurrent Get() and Set() operations.

func GetViper

func GetViper() *Viper

GetViper gets the global Viper instance.

func New

func New() *Viper

New returns an initialized Viper instance.

func NewWithOptions

func NewWithOptions(opts ...Option) *Viper

NewWithOptions creates a new Viper instance.

func Sub

func Sub(key string) *Viper

Sub returns new Viper instance representing a sub tree of this instance. Sub is case-insensitive for a key.

func (*Viper) AddConfigPath

func (v *Viper) AddConfigPath(in string)

AddConfigPath adds a path for Viper to search for the config file in. Can be called multiple times to define multiple search paths.

func (*Viper) AddRemoteProvider

func (v *Viper) AddRemoteProvider(provider, endpoint, path string) error

AddRemoteProvider adds a remote configuration source. Remote providers are merged in the order they are added. Providers added later have higher priority. provider is a string value: "etcd", "etcd3", "consul", "firestore" or "nats" are currently supported. endpoint is the url. etcd requires http://ip:port, consul requires ip:port, nats requires nats://ip:port path is the path in the k/v store to retrieve configuration To retrieve a config file called myapp.json from /configs/myapp.json you should set path to /configs and set config name (SetConfigName()) to "myapp".

func (*Viper) AddRemoteProviderCluster

func (v *Viper) AddRemoteProviderCluster(provider string, endpoints []string, path string) error

AddRemoteProviderCluster adds a remote configuration source with multiple endpoints. Providers added later have higher priority.

func (*Viper) AddSecureRemoteProvider

func (v *Viper) AddSecureRemoteProvider(provider, endpoint, path, secretkeyring string) error

AddSecureRemoteProvider adds a remote configuration source. Secure remote providers are merged in the order they are added. Providers added later have higher priority. provider is a string value: "etcd", "etcd3", "consul", "firestore" or "nats" are currently supported. endpoint is the url. etcd requires http://ip:port consul requires ip:port secretkeyring is the filepath to your openpgp secret keyring. e.g. /etc/secrets/myring.gpg path is the path in the k/v store to retrieve configuration To retrieve a config file called myapp.json from /configs/myapp.json you should set path to /configs and set config name (SetConfigName()) to "myapp". Secure Remote Providers are implemented with github.com/alkaid/crypt.

func (*Viper) AllKeys

func (v *Viper) AllKeys() []string

AllKeys returns all keys holding a value, regardless of where they are set. Nested keys are returned with a v.keyDelim separator.

func (*Viper) AllSettings

func (v *Viper) AllSettings() map[string]any

AllSettings merges all settings and returns them as a map[string]any.

func (*Viper) AllowEmptyEnv

func (v *Viper) AllowEmptyEnv(allowEmptyEnv bool)

AllowEmptyEnv tells Viper to consider set, but empty environment variables as valid values instead of falling back. For backward compatibility reasons this is false by default.

func (*Viper) AutomaticEnv

func (v *Viper) AutomaticEnv()

AutomaticEnv makes Viper check if environment variables match any of the existing keys (config, default or flags). If matching env vars are found, they are loaded into Viper.

func (*Viper) BindEnv

func (v *Viper) BindEnv(input ...string) error

BindEnv binds a Viper key to a ENV variable. ENV variables are case sensitive. If only a key is provided, it will use the env key matching the key, uppercased. If more arguments are provided, they will represent the env variable names that should bind to this key and will be taken in the specified order. EnvPrefix will be used when set when env name is not provided.

func (*Viper) BindFlagValue

func (v *Viper) BindFlagValue(key string, flag FlagValue) error

BindFlagValue binds a specific key to a FlagValue.

func (*Viper) BindFlagValues

func (v *Viper) BindFlagValues(flags FlagValueSet) (err error)

BindFlagValues binds a full FlagValue set to the configuration, using each flag's long name as the config key.

func (*Viper) BindPFlag

func (v *Viper) BindPFlag(key string, flag *pflag.Flag) error

BindPFlag binds a specific key to a pflag (as used by cobra). Example (where serverCmd is a Cobra instance):

serverCmd.Flags().Int("port", 1138, "Port to run Application server on")
Viper.BindPFlag("port", serverCmd.Flags().Lookup("port"))

func (*Viper) BindPFlags

func (v *Viper) BindPFlags(flags *pflag.FlagSet) error

BindPFlags binds a full flag set to the configuration, using each flag's long name as the config key.

func (*Viper) ConfigFileUsed

func (v *Viper) ConfigFileUsed() string

ConfigFileUsed returns the file used to populate the config registry.

func (*Viper) Debug

func (v *Viper) Debug()

Debug prints all configuration registries for debugging purposes.

func (*Viper) DebugTo

func (v *Viper) DebugTo(w io.Writer)

DebugTo writes all configuration registries to the given io.Writer.

func (*Viper) Get

func (v *Viper) Get(key string) any

Get retrieves the value associated with the key. Get is case-insensitive for a key. Get has the behavior of returning the value associated with the first place from where it is set. Viper will check in the following order: override, flag, env, config file, key/value store, default

Get returns an interface. For a specific value use one of the Get____ methods.

func (*Viper) GetBool

func (v *Viper) GetBool(key string) bool

GetBool returns the value associated with the key as a boolean.

func (*Viper) GetDuration

func (v *Viper) GetDuration(key string) time.Duration

GetDuration returns the value associated with the key as a duration.

func (*Viper) GetEnvPrefix

func (v *Viper) GetEnvPrefix() string

GetEnvPrefix returns the environment variable prefix.

func (*Viper) GetFloat64

func (v *Viper) GetFloat64(key string) float64

GetFloat64 returns the value associated with the key as a float64.

func (*Viper) GetInt

func (v *Viper) GetInt(key string) int

GetInt returns the value associated with the key as an integer.

func (*Viper) GetInt32 added in v1.1.0

func (v *Viper) GetInt32(key string) int32

GetInt32 returns the value associated with the key as an integer.

func (*Viper) GetInt64

func (v *Viper) GetInt64(key string) int64

GetInt64 returns the value associated with the key as an integer.

func (*Viper) GetIntSlice

func (v *Viper) GetIntSlice(key string) []int

GetIntSlice returns the value associated with the key as a slice of int values.

func (*Viper) GetSizeInBytes

func (v *Viper) GetSizeInBytes(key string) uint

GetSizeInBytes returns the size of the value associated with the given key in bytes.

func (*Viper) GetString

func (v *Viper) GetString(key string) string

GetString returns the value associated with the key as a string.

func (*Viper) GetStringMap

func (v *Viper) GetStringMap(key string) map[string]any

GetStringMap returns the value associated with the key as a map of interfaces.

func (*Viper) GetStringMapString

func (v *Viper) GetStringMapString(key string) map[string]string

GetStringMapString returns the value associated with the key as a map of strings.

func (*Viper) GetStringMapStringSlice

func (v *Viper) GetStringMapStringSlice(key string) map[string][]string

GetStringMapStringSlice returns the value associated with the key as a map to a slice of strings.

func (*Viper) GetStringSlice

func (v *Viper) GetStringSlice(key string) []string

GetStringSlice returns the value associated with the key as a slice of strings.

func (*Viper) GetTime

func (v *Viper) GetTime(key string) time.Time

GetTime returns the value associated with the key as time.

func (*Viper) GetUint

func (v *Viper) GetUint(key string) uint

GetUint returns the value associated with the key as an unsigned integer.

func (*Viper) GetUint8

func (v *Viper) GetUint8(key string) uint8

GetUint8 returns the value associated with the key as an unsigned integer.

func (*Viper) GetUint16

func (v *Viper) GetUint16(key string) uint16

GetUint16 returns the value associated with the key as an unsigned integer.

func (*Viper) GetUint32

func (v *Viper) GetUint32(key string) uint32

GetUint32 returns the value associated with the key as an unsigned integer.

func (*Viper) GetUint64

func (v *Viper) GetUint64(key string) uint64

GetUint64 returns the value associated with the key as an unsigned integer.

func (*Viper) InConfig

func (v *Viper) InConfig(key string) bool

InConfig checks to see if the given key (or an alias) is in the config file.

func (*Viper) IsSet

func (v *Viper) IsSet(key string) bool

IsSet checks to see if the key has been set in any of the data locations. IsSet is case-insensitive for a key.

func (*Viper) MergeConfig

func (v *Viper) MergeConfig(in io.Reader) error

MergeConfig merges a new configuration with an existing config.

func (*Viper) MergeConfigMap

func (v *Viper) MergeConfigMap(cfg map[string]any) error

MergeConfigMap merges the configuration from the map given with an existing config. Note that the map given may be modified.

func (*Viper) MergeInConfig

func (v *Viper) MergeInConfig() error

MergeInConfig merges a new configuration with an existing config.

func (*Viper) MustBindEnv

func (v *Viper) MustBindEnv(input ...string)

MustBindEnv wraps BindEnv in a panic. If there is an error binding an environment variable, MustBindEnv will panic.

func (*Viper) OnConfigChange

func (v *Viper) OnConfigChange(run func(in fsnotify.Event))

OnConfigChange sets the event handler that is called when a config file changes.

func (*Viper) ReadConfig

func (v *Viper) ReadConfig(in io.Reader) error

ReadConfig reads a configuration file, setting existing keys to nil if the key does not exist in the file.

func (*Viper) ReadInConfig

func (v *Viper) ReadInConfig() error

ReadInConfig will discover and load the configuration file from disk and key/value stores, searching in one of the defined paths.

func (*Viper) ReadRemoteConfig

func (v *Viper) ReadRemoteConfig() error

ReadRemoteConfig reads and merges all registered remote configuration sources. Providers added later override earlier providers.

func (*Viper) ReadRemoteConfigWithMerged

func (v *Viper) ReadRemoteConfigWithMerged(merged bool) error

ReadRemoteConfigWithMerged reads and optionally deep-merges all remote configuration sources.

func (*Viper) RegisterAlias

func (v *Viper) RegisterAlias(alias, key string)

RegisterAlias creates an alias that provides another accessor for the same key. This enables one to change a name without breaking the application.

func (*Viper) SafeWriteConfig added in v1.0.1

func (v *Viper) SafeWriteConfig() error

SafeWriteConfig writes current configuration to file only if the file does not exist.

func (*Viper) SafeWriteConfigAs added in v1.0.1

func (v *Viper) SafeWriteConfigAs(filename string) error

SafeWriteConfigAs writes current configuration to a given filename if it does not exist.

func (*Viper) Set

func (v *Viper) Set(key string, value any)

Set sets the value for the key in the override register. Set is case-insensitive for a key. Will be used instead of values obtained via flags, config file, ENV, default, or key/value store.

func (*Viper) SetConfigFile

func (v *Viper) SetConfigFile(in string)

SetConfigFile explicitly defines the path, name and extension of the config file. Viper will use this and not check any of the config paths.

func (*Viper) SetConfigName

func (v *Viper) SetConfigName(in string)

SetConfigName sets the name for the config file. Does not include extension.

func (*Viper) SetConfigPermissions

func (v *Viper) SetConfigPermissions(perm os.FileMode)

SetConfigPermissions sets the permissions for the config file.

func (*Viper) SetConfigType

func (v *Viper) SetConfigType(in string)

SetConfigType sets the type of the configuration returned by the remote source, e.g. "json".

func (*Viper) SetDefault

func (v *Viper) SetDefault(key string, value any)

SetDefault sets the default value for this key. SetDefault is case-insensitive for a key. Default only used when no value is provided by the user via flag, config or ENV.

func (*Viper) SetEnvKeyReplacer

func (v *Viper) SetEnvKeyReplacer(r *strings.Replacer)

SetEnvKeyReplacer sets the strings.Replacer on the viper object Useful for mapping an environmental variable to a key that does not match it.

func (*Viper) SetEnvPrefix

func (v *Viper) SetEnvPrefix(in string)

SetEnvPrefix defines a prefix that ENVIRONMENT variables will use. E.g. if your prefix is "spf", the env registry will look for env variables that start with "SPF_".

func (*Viper) SetFs

func (v *Viper) SetFs(fs afero.Fs)

SetFs sets the filesystem to use to read configuration.

func (*Viper) SetTypeByDefaultValue

func (v *Viper) SetTypeByDefaultValue(enable bool)

SetTypeByDefaultValue enables or disables the inference of a key value's type when the Get function is used based upon a key's default value as opposed to the value returned based on the normal fetch logic.

For example, if a key has a default value of []string{} and the same key is set via an environment variable to "a b c", a call to the Get function would return a string slice for the key if the key's type is inferred by the default value and the Get function would return:

[]string {"a", "b", "c"}

Otherwise the Get function would return:

"a b c"

func (*Viper) Sub

func (v *Viper) Sub(key string) *Viper

Sub returns a new Viper instance representing a sub tree of this instance. Sub is case-insensitive for a key.

func (*Viper) Unmarshal

func (v *Viper) Unmarshal(rawVal any, opts ...DecoderConfigOption) error

Unmarshal unmarshals the config into a Struct. Make sure that the tags on the fields of the structure are properly set.

func (*Viper) UnmarshalExact

func (v *Viper) UnmarshalExact(rawVal any, opts ...DecoderConfigOption) error

UnmarshalExact unmarshals the config into a Struct, erroring if a field is nonexistent in the destination struct.

func (*Viper) UnmarshalKey

func (v *Viper) UnmarshalKey(key string, rawVal any, opts ...DecoderConfigOption) error

UnmarshalKey takes a single key and unmarshals it into a Struct.

func (*Viper) WatchConfig

func (v *Viper) WatchConfig()

WatchConfig starts watching a config file for changes.

func (*Viper) WatchRemoteConfig

func (v *Viper) WatchRemoteConfig() error

WatchRemoteConfig updates configuration from available remote providers.

func (*Viper) WatchRemoteConfigOnChannel

func (v *Viper) WatchRemoteConfigOnChannel() error

WatchRemoteConfigOnChannel updates configuration from available remote providers.

func (*Viper) WatchRemoteConfigWithChannel

func (v *Viper) WatchRemoteConfigWithChannel(ctx context.Context, receiver chan *RemoteResponse, deepMerge bool) error

WatchRemoteConfigWithChannel watches every registered remote provider and forwards changes to receiver. The initial snapshot and watch startup are not atomic, so an update can occur between those operations.

func (*Viper) WriteConfig added in v1.0.1

func (v *Viper) WriteConfig() error

WriteConfig writes the current configuration to a file.

func (*Viper) WriteConfigAs added in v1.0.1

func (v *Viper) WriteConfigAs(filename string) error

WriteConfigAs writes current configuration to a given filename.

func (*Viper) WriteConfigTo

func (v *Viper) WriteConfigTo(w io.Writer) error

WriteConfigTo writes current configuration to an io.Writer.

Directories

Path Synopsis
internal
remote module

Jump to