Documentation
¶
Index ¶
- Variables
- func AddConfigPath(in string)
- func AddRemoteProvider(provider, endpoint, path string) error
- func AddSecureRemoteProvider(provider, endpoint, path, secretkeyring string) error
- func AllKeys() []string
- func AllSettings() map[string]any
- func AllowEmptyEnv(allowEmptyEnv bool)
- func AutomaticEnv()
- func BindEnv(input ...string) error
- func BindFlagValue(key string, flag FlagValue) error
- func BindFlagValues(flags FlagValueSet) error
- func BindPFlag(key string, flag *pflag.Flag) error
- func BindPFlags(flags *pflag.FlagSet) error
- func ConfigFileUsed() string
- func Debug()
- func DebugTo(w io.Writer)
- func Get(key string) any
- func GetBool(key string) bool
- func GetDuration(key string) time.Duration
- func GetEnvPrefix() string
- func GetFloat64(key string) float64
- func GetInt(key string) int
- func GetInt32(key string) int32
- func GetInt64(key string) int64
- func GetIntSlice(key string) []int
- func GetSizeInBytes(key string) uint
- func GetString(key string) string
- func GetStringMap(key string) map[string]any
- func GetStringMapString(key string) map[string]string
- func GetStringMapStringSlice(key string) map[string][]string
- func GetStringSlice(key string) []string
- func GetTime(key string) time.Time
- func GetUint(key string) uint
- func GetUint8(key string) uint8
- func GetUint16(key string) uint16
- func GetUint32(key string) uint32
- func GetUint64(key string) uint64
- func InConfig(key string) bool
- func IsSet(key string) bool
- func MergeConfig(in io.Reader) error
- func MergeConfigMap(cfg map[string]any) error
- func MergeInConfig() error
- func MustBindEnv(input ...string)
- func OnConfigChange(run func(in fsnotify.Event))
- func ReadConfig(in io.Reader) error
- func ReadInConfig() error
- func ReadRemoteConfig() error
- func ReadRemoteConfigWithMerged(merged bool) error
- func RegisterAlias(alias, key string)
- func Reset()
- func SafeWriteConfig() error
- func SafeWriteConfigAs(filename string) error
- func Set(key string, value any)
- func SetConfigFile(in string)
- func SetConfigName(in string)
- func SetConfigPermissions(perm os.FileMode)
- func SetConfigType(in string)
- func SetDefault(key string, value any)
- func SetEnvKeyReplacer(r *strings.Replacer)
- func SetEnvPrefix(in string)
- func SetFs(fs afero.Fs)
- func SetOptions(opts ...Option)
- func SetTypeByDefaultValue(enable bool)
- func Unmarshal(rawVal any, opts ...DecoderConfigOption) error
- func UnmarshalExact(rawVal any, opts ...DecoderConfigOption) error
- func UnmarshalKey(key string, rawVal any, opts ...DecoderConfigOption) error
- func WatchConfig()
- func WatchRemoteConfig() error
- func WriteConfig() error
- func WriteConfigAs(filename string) error
- func WriteConfigTo(w io.Writer) error
- type Codec
- type CodecRegistry
- type ConfigFileAlreadyExistsError
- type ConfigFileNotFoundErrordeprecated
- type ConfigMarshalError
- type ConfigParseError
- type Decoder
- type DecoderConfigOption
- type DecoderRegistry
- type DefaultCodecRegistry
- type Encoder
- type EncoderRegistry
- type FileLookupError
- type FileNotFoundError
- type FileNotFoundFromSearchError
- type Finder
- type FlagValue
- type FlagValueSet
- type Option
- func EnvKeyReplacer(r StringReplacer) Option
- func ExperimentalBindStruct() Option
- func ExperimentalFinder() Option
- func KeyDelimiter(d string) Option
- func WithCodecRegistry(r CodecRegistry) Option
- func WithDecodeHook(h mapstructure.DecodeHookFunc) Option
- func WithDecoderRegistry(r DecoderRegistry) Option
- func WithEncoderRegistry(r EncoderRegistry) Option
- func WithFinder(f Finder) Option
- func WithLogger(l *slog.Logger) Option
- type RemoteConfigError
- type RemoteProvider
- type RemoteResponse
- type StringReplacer
- type UnsupportedConfigError
- type UnsupportedRemoteProviderError
- type Viper
- func (v *Viper) AddConfigPath(in string)
- func (v *Viper) AddRemoteProvider(provider, endpoint, path string) error
- func (v *Viper) AddRemoteProviderCluster(provider string, endpoints []string, path string) error
- func (v *Viper) AddSecureRemoteProvider(provider, endpoint, path, secretkeyring string) error
- func (v *Viper) AllKeys() []string
- func (v *Viper) AllSettings() map[string]any
- func (v *Viper) AllowEmptyEnv(allowEmptyEnv bool)
- func (v *Viper) AutomaticEnv()
- func (v *Viper) BindEnv(input ...string) error
- func (v *Viper) BindFlagValue(key string, flag FlagValue) error
- func (v *Viper) BindFlagValues(flags FlagValueSet) (err error)
- func (v *Viper) BindPFlag(key string, flag *pflag.Flag) error
- func (v *Viper) BindPFlags(flags *pflag.FlagSet) error
- func (v *Viper) ConfigFileUsed() string
- func (v *Viper) Debug()
- func (v *Viper) DebugTo(w io.Writer)
- func (v *Viper) Get(key string) any
- func (v *Viper) GetBool(key string) bool
- func (v *Viper) GetDuration(key string) time.Duration
- func (v *Viper) GetEnvPrefix() string
- func (v *Viper) GetFloat64(key string) float64
- func (v *Viper) GetInt(key string) int
- func (v *Viper) GetInt32(key string) int32
- func (v *Viper) GetInt64(key string) int64
- func (v *Viper) GetIntSlice(key string) []int
- func (v *Viper) GetSizeInBytes(key string) uint
- func (v *Viper) GetString(key string) string
- func (v *Viper) GetStringMap(key string) map[string]any
- func (v *Viper) GetStringMapString(key string) map[string]string
- func (v *Viper) GetStringMapStringSlice(key string) map[string][]string
- func (v *Viper) GetStringSlice(key string) []string
- func (v *Viper) GetTime(key string) time.Time
- func (v *Viper) GetUint(key string) uint
- func (v *Viper) GetUint8(key string) uint8
- func (v *Viper) GetUint16(key string) uint16
- func (v *Viper) GetUint32(key string) uint32
- func (v *Viper) GetUint64(key string) uint64
- func (v *Viper) InConfig(key string) bool
- func (v *Viper) IsSet(key string) bool
- func (v *Viper) MergeConfig(in io.Reader) error
- func (v *Viper) MergeConfigMap(cfg map[string]any) error
- func (v *Viper) MergeInConfig() error
- func (v *Viper) MustBindEnv(input ...string)
- func (v *Viper) OnConfigChange(run func(in fsnotify.Event))
- func (v *Viper) ReadConfig(in io.Reader) error
- func (v *Viper) ReadInConfig() error
- func (v *Viper) ReadRemoteConfig() error
- func (v *Viper) ReadRemoteConfigWithMerged(merged bool) error
- func (v *Viper) RegisterAlias(alias, key string)
- func (v *Viper) SafeWriteConfig() error
- func (v *Viper) SafeWriteConfigAs(filename string) error
- func (v *Viper) Set(key string, value any)
- func (v *Viper) SetConfigFile(in string)
- func (v *Viper) SetConfigName(in string)
- func (v *Viper) SetConfigPermissions(perm os.FileMode)
- func (v *Viper) SetConfigType(in string)
- func (v *Viper) SetDefault(key string, value any)
- func (v *Viper) SetEnvKeyReplacer(r *strings.Replacer)
- func (v *Viper) SetEnvPrefix(in string)
- func (v *Viper) SetFs(fs afero.Fs)
- func (v *Viper) SetTypeByDefaultValue(enable bool)
- func (v *Viper) Sub(key string) *Viper
- func (v *Viper) Unmarshal(rawVal any, opts ...DecoderConfigOption) error
- func (v *Viper) UnmarshalExact(rawVal any, opts ...DecoderConfigOption) error
- func (v *Viper) UnmarshalKey(key string, rawVal any, opts ...DecoderConfigOption) error
- func (v *Viper) WatchConfig()
- func (v *Viper) WatchRemoteConfig() error
- func (v *Viper) WatchRemoteConfigOnChannel() error
- func (v *Viper) WatchRemoteConfigWithChannel(ctx context.Context, receiver chan *RemoteResponse, deepMerge bool) error
- func (v *Viper) WriteConfig() error
- func (v *Viper) WriteConfigAs(filename string) error
- func (v *Viper) WriteConfigTo(w io.Writer) error
Examples ¶
Constants ¶
This section is empty.
Variables ¶
var RemoteConfig remoteConfigFactory
RemoteConfig is optional, see the remote package.
var SupportedExts = []string{"json", "toml", "yaml", "yml", "properties", "props", "prop", "hcl", "tfvars", "dotenv", "env", "ini"}
SupportedExts are universally supported extensions.
var SupportedRemoteProviders = []string{"etcd", "etcd3", "consul", "firestore", "nats"}
SupportedRemoteProviders are universally supported remote providers.
Functions ¶
func AddConfigPath ¶
func AddConfigPath(in string)
AddConfigPath adds a path for Viper to search for the config file in. Can be called multiple times to define multiple search paths.
func AddRemoteProvider ¶
AddRemoteProvider adds a remote configuration source. Remote providers are merged in the order they are added. Providers added later have higher priority. provider is a string value: "etcd", "etcd3", "consul", "firestore" or "nats" are currently supported. endpoint is the url. etcd requires http://ip:port, consul requires ip:port, nats requires nats://ip:port path is the path in the k/v store to retrieve configuration To retrieve a config file called myapp.json from /configs/myapp.json you should set path to /configs and set config name (SetConfigName()) to "myapp".
func AddSecureRemoteProvider ¶
AddSecureRemoteProvider adds a remote configuration source. Secure remote providers are merged in the order they are added. Providers added later have higher priority. provider is a string value: "etcd", "etcd3", "consul", "firestore" or "nats" are currently supported. endpoint is the url. etcd requires http://ip:port consul requires ip:port secretkeyring is the filepath to your openpgp secret keyring. e.g. /etc/secrets/myring.gpg path is the path in the k/v store to retrieve configuration To retrieve a config file called myapp.json from /configs/myapp.json you should set path to /configs and set config name (SetConfigName()) to "myapp". Secure Remote Providers are implemented with github.com/alkaid/crypt.
func AllKeys ¶
func AllKeys() []string
AllKeys returns all keys holding a value, regardless of where they are set. Nested keys are returned with a v.keyDelim separator.
func AllSettings ¶
AllSettings merges all settings and returns them as a map[string]any.
func AllowEmptyEnv ¶
func AllowEmptyEnv(allowEmptyEnv bool)
AllowEmptyEnv tells Viper to consider set, but empty environment variables as valid values instead of falling back. For backward compatibility reasons this is false by default.
func AutomaticEnv ¶
func AutomaticEnv()
AutomaticEnv makes Viper check if environment variables match any of the existing keys (config, default or flags). If matching env vars are found, they are loaded into Viper.
func BindEnv ¶
BindEnv binds a Viper key to a ENV variable. ENV variables are case sensitive. If only a key is provided, it will use the env key matching the key, uppercased. If more arguments are provided, they will represent the env variable names that should bind to this key and will be taken in the specified order. EnvPrefix will be used when set when env name is not provided.
func BindFlagValue ¶
BindFlagValue binds a specific key to a FlagValue.
func BindFlagValues ¶
func BindFlagValues(flags FlagValueSet) error
BindFlagValues binds a full FlagValue set to the configuration, using each flag's long name as the config key.
func BindPFlag ¶
BindPFlag binds a specific key to a pflag (as used by cobra). Example (where serverCmd is a Cobra instance):
serverCmd.Flags().Int("port", 1138, "Port to run Application server on")
Viper.BindPFlag("port", serverCmd.Flags().Lookup("port"))
func BindPFlags ¶
BindPFlags binds a full flag set to the configuration, using each flag's long name as the config key.
func ConfigFileUsed ¶
func ConfigFileUsed() string
ConfigFileUsed returns the file used to populate the config registry.
func Get ¶
Get can retrieve any value given the key to use. Get is case-insensitive for a key. Get has the behavior of returning the value associated with the first place from where it is set. Viper will check in the following order: override, flag, env, config file, key/value store, default
Get returns an interface. For a specific value use one of the Get____ methods.
func GetDuration ¶
GetDuration returns the value associated with the key as a duration.
func GetEnvPrefix ¶
func GetEnvPrefix() string
GetEnvPrefix returns the environment variable prefix.
func GetFloat64 ¶
GetFloat64 returns the value associated with the key as a float64.
func GetIntSlice ¶
GetIntSlice returns the value associated with the key as a slice of int values.
func GetSizeInBytes ¶
GetSizeInBytes returns the size of the value associated with the given key in bytes.
func GetStringMap ¶
GetStringMap returns the value associated with the key as a map of interfaces.
func GetStringMapString ¶
GetStringMapString returns the value associated with the key as a map of strings.
func GetStringMapStringSlice ¶
GetStringMapStringSlice returns the value associated with the key as a map to a slice of strings.
func GetStringSlice ¶
GetStringSlice returns the value associated with the key as a slice of strings.
func IsSet ¶
IsSet checks to see if the key has been set in any of the data locations. IsSet is case-insensitive for a key.
func MergeConfig ¶
MergeConfig merges a new configuration with an existing config.
func MergeConfigMap ¶
MergeConfigMap merges the configuration from the map given with an existing config. Note that the map given may be modified.
func MergeInConfig ¶
func MergeInConfig() error
MergeInConfig merges a new configuration with an existing config.
func MustBindEnv ¶
func MustBindEnv(input ...string)
MustBindEnv wraps BindEnv in a panic. If there is an error binding an environment variable, MustBindEnv will panic.
func OnConfigChange ¶
OnConfigChange sets the event handler that is called when a config file changes.
func ReadConfig ¶
ReadConfig will read a configuration file, setting existing keys to nil if the key does not exist in the file.
func ReadInConfig ¶
func ReadInConfig() error
ReadInConfig will discover and load the configuration file from disk and key/value stores, searching in one of the defined paths.
func ReadRemoteConfig ¶
func ReadRemoteConfig() error
ReadRemoteConfig reads and merges all registered remote configuration sources. Providers added later override earlier providers.
func ReadRemoteConfigWithMerged ¶
ReadRemoteConfigWithMerged reads and optionally deep-merges all remote configuration sources.
func RegisterAlias ¶
func RegisterAlias(alias, key string)
RegisterAlias creates an alias that provides another accessor for the same key. This enables one to change a name without breaking the application.
func Reset ¶
func Reset()
Reset is intended for testing, will reset all to default settings. In the public interface for the viper package so applications can use it in their testing as well.
func SafeWriteConfig ¶ added in v1.0.1
func SafeWriteConfig() error
SafeWriteConfig writes current configuration to file only if the file does not exist.
func SafeWriteConfigAs ¶ added in v1.0.1
SafeWriteConfigAs writes current configuration to a given filename if it does not exist.
func Set ¶
Set sets the value for the key in the override register. Set is case-insensitive for a key. Will be used instead of values obtained via flags, config file, ENV, default, or key/value store.
func SetConfigFile ¶
func SetConfigFile(in string)
SetConfigFile explicitly defines the path, name and extension of the config file. Viper will use this and not check any of the config paths.
func SetConfigName ¶
func SetConfigName(in string)
SetConfigName sets name for the config file. Does not include extension.
func SetConfigPermissions ¶
SetConfigPermissions sets the permissions for the config file.
func SetConfigType ¶
func SetConfigType(in string)
SetConfigType sets the type of the configuration returned by the remote source, e.g. "json".
func SetDefault ¶
SetDefault sets the default value for this key. SetDefault is case-insensitive for a key. Default only used when no value is provided by the user via flag, config or ENV.
func SetEnvKeyReplacer ¶
SetEnvKeyReplacer sets the strings.Replacer on the viper object Useful for mapping an environmental variable to a key that does not match it.
func SetEnvPrefix ¶
func SetEnvPrefix(in string)
SetEnvPrefix defines a prefix that ENVIRONMENT variables will use. E.g. if your prefix is "spf", the env registry will look for env variables that start with "SPF_".
func SetOptions ¶
func SetOptions(opts ...Option)
SetOptions sets the options on the global Viper instance.
Be careful when using this function: subsequent calls may override options you set. It's always better to use a local Viper instance.
func SetTypeByDefaultValue ¶
func SetTypeByDefaultValue(enable bool)
SetTypeByDefaultValue enables or disables the inference of a key value's type when the Get function is used based upon a key's default value as opposed to the value returned based on the normal fetch logic.
For example, if a key has a default value of []string{} and the same key is set via an environment variable to "a b c", a call to the Get function would return a string slice for the key if the key's type is inferred by the default value and the Get function would return:
[]string {"a", "b", "c"}
Otherwise the Get function would return:
"a b c"
func Unmarshal ¶
func Unmarshal(rawVal any, opts ...DecoderConfigOption) error
Unmarshal unmarshals the config into a Struct. Make sure that the tags on the fields of the structure are properly set.
func UnmarshalExact ¶
func UnmarshalExact(rawVal any, opts ...DecoderConfigOption) error
UnmarshalExact unmarshals the config into a Struct, erroring if a field is nonexistent in the destination struct.
func UnmarshalKey ¶
func UnmarshalKey(key string, rawVal any, opts ...DecoderConfigOption) error
UnmarshalKey takes a single key and unmarshals it into a Struct.
func WatchRemoteConfig ¶
func WatchRemoteConfig() error
WatchRemoteConfig updates configuration from available remote providers.
func WriteConfig ¶ added in v1.0.1
func WriteConfig() error
WriteConfig writes the current configuration to a file.
func WriteConfigAs ¶ added in v1.0.1
WriteConfigAs writes current configuration to a given filename.
Types ¶
type CodecRegistry ¶
type CodecRegistry interface {
EncoderRegistry
DecoderRegistry
}
CodecRegistry combines EncoderRegistry and DecoderRegistry interfaces.
type ConfigFileAlreadyExistsError ¶
type ConfigFileAlreadyExistsError string
ConfigFileAlreadyExistsError denotes failure to write new configuration file.
func (ConfigFileAlreadyExistsError) Error ¶
func (e ConfigFileAlreadyExistsError) Error() string
Error returns the formatted error when configuration already exists.
type ConfigFileNotFoundError
deprecated
type ConfigFileNotFoundError struct {
// contains filtered or unexported fields
}
ConfigFileNotFoundError denotes failing to find a configuration file from a search.
Deprecated: This is error wraps FileNotFoundFromSearchError, which should be used instead.
func (ConfigFileNotFoundError) Error ¶
func (e ConfigFileNotFoundError) Error() string
Error returns the formatted error.
func (ConfigFileNotFoundError) Unwrap ¶
func (e ConfigFileNotFoundError) Unwrap() error
Unwraps to FileNotFoundFromSearchError.
type ConfigMarshalError ¶ added in v1.0.1
type ConfigMarshalError struct {
// contains filtered or unexported fields
}
ConfigMarshalError happens when failing to marshal the configuration.
func (ConfigMarshalError) Error ¶ added in v1.0.1
func (e ConfigMarshalError) Error() string
Error returns the formatted configuration error.
type ConfigParseError ¶
type ConfigParseError struct {
// contains filtered or unexported fields
}
ConfigParseError denotes failing to parse configuration file.
func (ConfigParseError) Error ¶
func (pe ConfigParseError) Error() string
Error returns the formatted configuration error.
func (ConfigParseError) Unwrap ¶
func (pe ConfigParseError) Unwrap() error
Unwrap returns the wrapped error.
type Decoder ¶
Decoder decodes the contents of a byte slice into Viper's internal data structures. It's primarily used for decoding contents of a file into a map[string]any.
type DecoderConfigOption ¶ added in v1.1.0
type DecoderConfigOption func(*mapstructure.DecoderConfig)
A DecoderConfigOption can be passed to viper.Unmarshal to configure mapstructure.DecoderConfig options.
func DecodeHook ¶ added in v1.1.0
func DecodeHook(hook mapstructure.DecodeHookFunc) DecoderConfigOption
DecodeHook returns a DecoderConfigOption which overrides the default DecoderConfig.DecodeHook value, the default is:
mapstructure.ComposeDecodeHookFunc(
mapstructure.StringToTimeDurationHookFunc(),
mapstructure.StringToSliceHookFunc(","),
)
type DecoderRegistry ¶
DecoderRegistry returns an Decoder for a given format.
Format is case-insensitive.
DecoderRegistry returns an error if no Decoder is registered for the format.
type DefaultCodecRegistry ¶
type DefaultCodecRegistry struct {
// contains filtered or unexported fields
}
DefaultCodecRegistry is a simple implementation of CodecRegistry that allows registering custom [Codec]s.
func NewCodecRegistry ¶
func NewCodecRegistry() *DefaultCodecRegistry
NewCodecRegistry returns a new CodecRegistry, ready to accept custom [Codec]s.
func (*DefaultCodecRegistry) Decoder ¶
func (r *DefaultCodecRegistry) Decoder(format string) (Decoder, error)
Decoder implements the DecoderRegistry interface.
Format is case-insensitive.
func (*DefaultCodecRegistry) Encoder ¶
func (r *DefaultCodecRegistry) Encoder(format string) (Encoder, error)
Encoder implements the EncoderRegistry interface.
Format is case-insensitive.
func (*DefaultCodecRegistry) RegisterCodec ¶
func (r *DefaultCodecRegistry) RegisterCodec(format string, codec Codec) error
RegisterCodec registers a custom Codec.
Format is case-insensitive.
type Encoder ¶
Encoder encodes Viper's internal data structures into a byte representation. It's primarily used for encoding a map[string]any into a file format.
type EncoderRegistry ¶
EncoderRegistry returns an Encoder for a given format.
Format is case-insensitive.
EncoderRegistry returns an error if no Encoder is registered for the format.
type FileLookupError ¶
type FileLookupError interface {
error
// contains filtered or unexported methods
}
FileLookupError is returned when Viper cannot resolve a configuration file.
This is meant to be a common interface for all file look-up errors, occurring either because a file does not exist or because it cannot find any file matching finder criteria.
type FileNotFoundError ¶
type FileNotFoundError struct {
// contains filtered or unexported fields
}
FileNotFoundError denotes failing to find a specific configuration file.
func (FileNotFoundError) Error ¶
func (e FileNotFoundError) Error() string
Error returns the formatted error.
type FileNotFoundFromSearchError ¶
type FileNotFoundFromSearchError struct {
// contains filtered or unexported fields
}
FileNotFoundFromSearchError denotes failing to find a configuration file from a search. Wraps ConfigFileNotFoundError.
func (FileNotFoundFromSearchError) Error ¶
func (e FileNotFoundFromSearchError) Error() string
Error returns the formatted error.
type Finder ¶
Finder looks for files and directories in an afero.Fs filesystem.
Example ¶
package main
import (
"fmt"
"github.com/sagikazarmark/locafero"
"github.com/spf13/afero"
"github.com/alkaid/viper"
)
func main() {
fs := afero.NewMemMapFs()
fs.Mkdir("/home/user", 0o777)
f, _ := fs.Create("/home/user/myapp.yaml")
f.WriteString("foo: bar")
f.Close()
// HCL will have a "lower" priority in the search order
fs.Create("/home/user/myapp.hcl")
finder := locafero.Finder{
Paths: []string{"/home/user"},
Names: locafero.NameWithExtensions("myapp", viper.SupportedExts...),
Type: locafero.FileTypeFile, // This is important!
}
v := viper.NewWithOptions(viper.WithFinder(finder))
v.SetFs(fs)
v.ReadInConfig()
fmt.Println(v.GetString("foo"))
}
Output: bar
func Finders ¶
Finders combines multiple finders into one.
Example ¶
package main
import (
"fmt"
"github.com/sagikazarmark/locafero"
"github.com/spf13/afero"
"github.com/alkaid/viper"
)
func main() {
fs := afero.NewMemMapFs()
fs.Mkdir("/home/user", 0o777)
f, _ := fs.Create("/home/user/myapp.yaml")
f.WriteString("foo: bar")
f.Close()
fs.Mkdir("/etc/myapp", 0o777)
fs.Create("/etc/myapp/config.yaml")
// Combine multiple finders to search for files in multiple locations with different criteria
finder := viper.Finders(
locafero.Finder{
Paths: []string{"/home/user"},
Names: locafero.NameWithExtensions("myapp", viper.SupportedExts...),
Type: locafero.FileTypeFile, // This is important!
},
locafero.Finder{
Paths: []string{"/etc/myapp"},
Names: []string{"config.yaml"}, // Only accept YAML files in the system config directory
Type: locafero.FileTypeFile, // This is important!
},
)
v := viper.NewWithOptions(viper.WithFinder(finder))
v.SetFs(fs)
v.ReadInConfig()
fmt.Println(v.GetString("foo"))
}
Output: bar
type FlagValue ¶
type FlagValue interface {
HasChanged() bool
Name() string
ValueString() string
ValueType() string
}
FlagValue is an interface that users can implement to bind different flags to viper.
type FlagValueSet ¶
type FlagValueSet interface {
VisitAll(fn func(FlagValue))
}
FlagValueSet is an interface that users can implement to bind a set of flags to viper.
type Option ¶
type Option interface {
// contains filtered or unexported methods
}
Option configures Viper using the functional options paradigm popularized by Rob Pike and Dave Cheney. If you're unfamiliar with this style, see https://commandcenter.blogspot.com/2014/01/self-referential-functions-and-design.html and https://dave.cheney.net/2014/10/17/functional-options-for-friendly-apis.
func EnvKeyReplacer ¶
func EnvKeyReplacer(r StringReplacer) Option
EnvKeyReplacer sets a replacer used for mapping environment variables to internal keys.
func ExperimentalBindStruct ¶
func ExperimentalBindStruct() Option
ExperimentalBindStruct tells Viper to use the new bind struct feature.
func ExperimentalFinder ¶
func ExperimentalFinder() Option
ExperimentalFinder tells Viper to use the new Finder interface for finding configuration files.
func KeyDelimiter ¶
KeyDelimiter sets the delimiter used for determining key parts. By default it's value is ".".
func WithCodecRegistry ¶
func WithCodecRegistry(r CodecRegistry) Option
WithCodecRegistry sets a custom EncoderRegistry and DecoderRegistry.
func WithDecodeHook ¶
func WithDecodeHook(h mapstructure.DecodeHookFunc) Option
WithDecodeHook sets a default decode hook for mapstructure.
func WithDecoderRegistry ¶
func WithDecoderRegistry(r DecoderRegistry) Option
WithDecoderRegistry sets a custom DecoderRegistry.
func WithEncoderRegistry ¶
func WithEncoderRegistry(r EncoderRegistry) Option
WithEncoderRegistry sets a custom EncoderRegistry.
type RemoteConfigError ¶
type RemoteConfigError string
RemoteConfigError denotes encountering an error while trying to pull the configuration from the remote provider.
func (RemoteConfigError) Error ¶
func (rce RemoteConfigError) Error() string
Error returns the formatted remote provider error.
type RemoteProvider ¶
type RemoteProvider interface {
Provider() string
Endpoint() string
Endpoints() []string
Path() string
SecretKeyring() string
}
RemoteProvider stores the configuration necessary to connect to a remote key/value store. Optional secretKeyring to unencrypt encrypted values can be provided.
type RemoteResponse ¶
RemoteResponse represents a response from a remote configuration provider.
type StringReplacer ¶
type StringReplacer interface {
// Replace returns a copy of s with all replacements performed.
Replace(s string) string
}
StringReplacer applies a set of replacements to a string.
type UnsupportedConfigError ¶
type UnsupportedConfigError string
UnsupportedConfigError denotes encountering an unsupported configuration filetype.
func (UnsupportedConfigError) Error ¶
func (str UnsupportedConfigError) Error() string
Error returns the formatted configuration error.
type UnsupportedRemoteProviderError ¶
type UnsupportedRemoteProviderError string
UnsupportedRemoteProviderError denotes encountering an unsupported remote provider. Currently only etcd and Consul are supported.
func (UnsupportedRemoteProviderError) Error ¶
func (str UnsupportedRemoteProviderError) Error() string
Error returns the formatted remote provider error.
type Viper ¶
type Viper struct {
// contains filtered or unexported fields
}
Viper is a prioritized configuration registry. It maintains a set of configuration sources, fetches values to populate those, and provides them according to the source's priority. The priority of the sources is the following: 1. overrides 2. flags 3. env. variables 4. config file 5. key/value store 6. defaults
For example, if values from the following sources were loaded:
Defaults : {
"secret": "",
"user": "default",
"endpoint": "https://localhost"
}
Config : {
"user": "root"
"secret": "defaultsecret"
}
Env : {
"secret": "somesecretkey"
}
The resulting config will have the following values:
{
"secret": "somesecretkey",
"user": "root",
"endpoint": "https://localhost"
}
Note: Vipers are not safe for concurrent Get() and Set() operations.
func NewWithOptions ¶
NewWithOptions creates a new Viper instance.
func Sub ¶
Sub returns new Viper instance representing a sub tree of this instance. Sub is case-insensitive for a key.
func (*Viper) AddConfigPath ¶
AddConfigPath adds a path for Viper to search for the config file in. Can be called multiple times to define multiple search paths.
func (*Viper) AddRemoteProvider ¶
AddRemoteProvider adds a remote configuration source. Remote providers are merged in the order they are added. Providers added later have higher priority. provider is a string value: "etcd", "etcd3", "consul", "firestore" or "nats" are currently supported. endpoint is the url. etcd requires http://ip:port, consul requires ip:port, nats requires nats://ip:port path is the path in the k/v store to retrieve configuration To retrieve a config file called myapp.json from /configs/myapp.json you should set path to /configs and set config name (SetConfigName()) to "myapp".
func (*Viper) AddRemoteProviderCluster ¶
AddRemoteProviderCluster adds a remote configuration source with multiple endpoints. Providers added later have higher priority.
func (*Viper) AddSecureRemoteProvider ¶
AddSecureRemoteProvider adds a remote configuration source. Secure remote providers are merged in the order they are added. Providers added later have higher priority. provider is a string value: "etcd", "etcd3", "consul", "firestore" or "nats" are currently supported. endpoint is the url. etcd requires http://ip:port consul requires ip:port secretkeyring is the filepath to your openpgp secret keyring. e.g. /etc/secrets/myring.gpg path is the path in the k/v store to retrieve configuration To retrieve a config file called myapp.json from /configs/myapp.json you should set path to /configs and set config name (SetConfigName()) to "myapp". Secure Remote Providers are implemented with github.com/alkaid/crypt.
func (*Viper) AllKeys ¶
AllKeys returns all keys holding a value, regardless of where they are set. Nested keys are returned with a v.keyDelim separator.
func (*Viper) AllSettings ¶
AllSettings merges all settings and returns them as a map[string]any.
func (*Viper) AllowEmptyEnv ¶
AllowEmptyEnv tells Viper to consider set, but empty environment variables as valid values instead of falling back. For backward compatibility reasons this is false by default.
func (*Viper) AutomaticEnv ¶
func (v *Viper) AutomaticEnv()
AutomaticEnv makes Viper check if environment variables match any of the existing keys (config, default or flags). If matching env vars are found, they are loaded into Viper.
func (*Viper) BindEnv ¶
BindEnv binds a Viper key to a ENV variable. ENV variables are case sensitive. If only a key is provided, it will use the env key matching the key, uppercased. If more arguments are provided, they will represent the env variable names that should bind to this key and will be taken in the specified order. EnvPrefix will be used when set when env name is not provided.
func (*Viper) BindFlagValue ¶
BindFlagValue binds a specific key to a FlagValue.
func (*Viper) BindFlagValues ¶
func (v *Viper) BindFlagValues(flags FlagValueSet) (err error)
BindFlagValues binds a full FlagValue set to the configuration, using each flag's long name as the config key.
func (*Viper) BindPFlag ¶
BindPFlag binds a specific key to a pflag (as used by cobra). Example (where serverCmd is a Cobra instance):
serverCmd.Flags().Int("port", 1138, "Port to run Application server on")
Viper.BindPFlag("port", serverCmd.Flags().Lookup("port"))
func (*Viper) BindPFlags ¶
BindPFlags binds a full flag set to the configuration, using each flag's long name as the config key.
func (*Viper) ConfigFileUsed ¶
ConfigFileUsed returns the file used to populate the config registry.
func (*Viper) Debug ¶
func (v *Viper) Debug()
Debug prints all configuration registries for debugging purposes.
func (*Viper) Get ¶
Get retrieves the value associated with the key. Get is case-insensitive for a key. Get has the behavior of returning the value associated with the first place from where it is set. Viper will check in the following order: override, flag, env, config file, key/value store, default
Get returns an interface. For a specific value use one of the Get____ methods.
func (*Viper) GetDuration ¶
GetDuration returns the value associated with the key as a duration.
func (*Viper) GetEnvPrefix ¶
GetEnvPrefix returns the environment variable prefix.
func (*Viper) GetFloat64 ¶
GetFloat64 returns the value associated with the key as a float64.
func (*Viper) GetInt32 ¶ added in v1.1.0
GetInt32 returns the value associated with the key as an integer.
func (*Viper) GetIntSlice ¶
GetIntSlice returns the value associated with the key as a slice of int values.
func (*Viper) GetSizeInBytes ¶
GetSizeInBytes returns the size of the value associated with the given key in bytes.
func (*Viper) GetStringMap ¶
GetStringMap returns the value associated with the key as a map of interfaces.
func (*Viper) GetStringMapString ¶
GetStringMapString returns the value associated with the key as a map of strings.
func (*Viper) GetStringMapStringSlice ¶
GetStringMapStringSlice returns the value associated with the key as a map to a slice of strings.
func (*Viper) GetStringSlice ¶
GetStringSlice returns the value associated with the key as a slice of strings.
func (*Viper) GetUint16 ¶
GetUint16 returns the value associated with the key as an unsigned integer.
func (*Viper) GetUint32 ¶
GetUint32 returns the value associated with the key as an unsigned integer.
func (*Viper) GetUint64 ¶
GetUint64 returns the value associated with the key as an unsigned integer.
func (*Viper) InConfig ¶
InConfig checks to see if the given key (or an alias) is in the config file.
func (*Viper) IsSet ¶
IsSet checks to see if the key has been set in any of the data locations. IsSet is case-insensitive for a key.
func (*Viper) MergeConfig ¶
MergeConfig merges a new configuration with an existing config.
func (*Viper) MergeConfigMap ¶
MergeConfigMap merges the configuration from the map given with an existing config. Note that the map given may be modified.
func (*Viper) MergeInConfig ¶
MergeInConfig merges a new configuration with an existing config.
func (*Viper) MustBindEnv ¶
MustBindEnv wraps BindEnv in a panic. If there is an error binding an environment variable, MustBindEnv will panic.
func (*Viper) OnConfigChange ¶
OnConfigChange sets the event handler that is called when a config file changes.
func (*Viper) ReadConfig ¶
ReadConfig reads a configuration file, setting existing keys to nil if the key does not exist in the file.
func (*Viper) ReadInConfig ¶
ReadInConfig will discover and load the configuration file from disk and key/value stores, searching in one of the defined paths.
func (*Viper) ReadRemoteConfig ¶
ReadRemoteConfig reads and merges all registered remote configuration sources. Providers added later override earlier providers.
func (*Viper) ReadRemoteConfigWithMerged ¶
ReadRemoteConfigWithMerged reads and optionally deep-merges all remote configuration sources.
func (*Viper) RegisterAlias ¶
RegisterAlias creates an alias that provides another accessor for the same key. This enables one to change a name without breaking the application.
func (*Viper) SafeWriteConfig ¶ added in v1.0.1
SafeWriteConfig writes current configuration to file only if the file does not exist.
func (*Viper) SafeWriteConfigAs ¶ added in v1.0.1
SafeWriteConfigAs writes current configuration to a given filename if it does not exist.
func (*Viper) Set ¶
Set sets the value for the key in the override register. Set is case-insensitive for a key. Will be used instead of values obtained via flags, config file, ENV, default, or key/value store.
func (*Viper) SetConfigFile ¶
SetConfigFile explicitly defines the path, name and extension of the config file. Viper will use this and not check any of the config paths.
func (*Viper) SetConfigName ¶
SetConfigName sets the name for the config file. Does not include extension.
func (*Viper) SetConfigPermissions ¶
SetConfigPermissions sets the permissions for the config file.
func (*Viper) SetConfigType ¶
SetConfigType sets the type of the configuration returned by the remote source, e.g. "json".
func (*Viper) SetDefault ¶
SetDefault sets the default value for this key. SetDefault is case-insensitive for a key. Default only used when no value is provided by the user via flag, config or ENV.
func (*Viper) SetEnvKeyReplacer ¶
SetEnvKeyReplacer sets the strings.Replacer on the viper object Useful for mapping an environmental variable to a key that does not match it.
func (*Viper) SetEnvPrefix ¶
SetEnvPrefix defines a prefix that ENVIRONMENT variables will use. E.g. if your prefix is "spf", the env registry will look for env variables that start with "SPF_".
func (*Viper) SetTypeByDefaultValue ¶
SetTypeByDefaultValue enables or disables the inference of a key value's type when the Get function is used based upon a key's default value as opposed to the value returned based on the normal fetch logic.
For example, if a key has a default value of []string{} and the same key is set via an environment variable to "a b c", a call to the Get function would return a string slice for the key if the key's type is inferred by the default value and the Get function would return:
[]string {"a", "b", "c"}
Otherwise the Get function would return:
"a b c"
func (*Viper) Sub ¶
Sub returns a new Viper instance representing a sub tree of this instance. Sub is case-insensitive for a key.
func (*Viper) Unmarshal ¶
func (v *Viper) Unmarshal(rawVal any, opts ...DecoderConfigOption) error
Unmarshal unmarshals the config into a Struct. Make sure that the tags on the fields of the structure are properly set.
func (*Viper) UnmarshalExact ¶
func (v *Viper) UnmarshalExact(rawVal any, opts ...DecoderConfigOption) error
UnmarshalExact unmarshals the config into a Struct, erroring if a field is nonexistent in the destination struct.
func (*Viper) UnmarshalKey ¶
func (v *Viper) UnmarshalKey(key string, rawVal any, opts ...DecoderConfigOption) error
UnmarshalKey takes a single key and unmarshals it into a Struct.
func (*Viper) WatchConfig ¶
func (v *Viper) WatchConfig()
WatchConfig starts watching a config file for changes.
func (*Viper) WatchRemoteConfig ¶
WatchRemoteConfig updates configuration from available remote providers.
func (*Viper) WatchRemoteConfigOnChannel ¶
WatchRemoteConfigOnChannel updates configuration from available remote providers.
func (*Viper) WatchRemoteConfigWithChannel ¶
func (v *Viper) WatchRemoteConfigWithChannel(ctx context.Context, receiver chan *RemoteResponse, deepMerge bool) error
WatchRemoteConfigWithChannel watches every registered remote provider and forwards changes to receiver. The initial snapshot and watch startup are not atomic, so an update can occur between those operations.
func (*Viper) WriteConfig ¶ added in v1.0.1
WriteConfig writes the current configuration to a file.
func (*Viper) WriteConfigAs ¶ added in v1.0.1
WriteConfigAs writes current configuration to a given filename.